local inCombat = nil
local dndBanner = "<SBW> "

local dndString = dndBanner .. "I'm busy fighting %s (currently at %d%%). You'll be notified when combat ends."
local combatEndedString = dndBanner .. "Combat ended after %d minutes."

do
	local locale = GetLocale()
	if locale == "deDE" then
		dndString = dndBanner .. "Ich bin im Kampf mit %s (noch %d%% Leben). Du wirst nach Kampfende benachrichtigt werden."
		combatEndedString = dndBanner .. "Kampf beendet nach %d Minuten."
	end
end

local dndBannerLen = dndBanner:len()

local whisperers = {}
local combatStart = nil
local boss = nil
local bossHp = nil

local pname = UnitName("player")
local orig = ChatFrame_MessageEventHandler
ChatFrame_MessageEventHandler = function(event, ...)
	if type(arg2) ~= "string" or arg2 == pname or
	  (UnitExists("target") and UnitName("target") == arg2) or
	  (type(arg6) == "string" and arg6 == "GM") then return orig(event, ...) end

	local banner = arg1:sub(1, dndBannerLen)
	if event == "CHAT_MSG_WHISPER_INFORM" and banner == dndBanner then
		-- Hide AddOns outgoing whispers
		return
	elseif event == "CHAT_MSG_WHISPER" and boss and banner ~= dndBanner and (not whisperers[arg2] or whisperers[arg2] == 1) then
		-- Let him know we are fighting a boss
		whisperers[arg2] = 2
		SendChatMessage(dndString:format(boss, bossHp), "WHISPER", nil, arg2)
	end

	return orig(event, ...)
end

local function checkTarget(id)
	return UnitExists(id) and UnitAffectingCombat(id) and UnitClassification(id) == "worldboss"
end

local function scan()
	if checkTarget("target") then return "target" end
	if checkTarget("focus") then return "focus" end
	if UnitInRaid("player") then
		for i = 1, 40 do
			if checkTarget("raid"..i.."target") then return "raid"..i.."target" end
		end
	else
		for i = 1, 5 do
			if checkTarget("party"..i.."target") then return "party"..i.."target" end
		end
	end
end

local frame = CreateFrame("Frame")
local function updateBossTarget()
	local target = scan()
	if not target then return end
	if not boss then
		frame:RegisterEvent("UNIT_HEALTH")
	end
	boss = UnitName(target)
	bossHp = UnitHealth(target)
end

frame:SetScript("OnEvent", function()
	if event == "PLAYER_REGEN_DISABLED" then
		frame:RegisterEvent("PLAYER_TARGET_CHANGED")
		combatStart = GetTime()
		updateBossTarget()
	elseif event == "PLAYER_TARGET_CHANGED" then
		updateBossTarget()
	elseif event == "PLAYER_REGEN_ENABLED" then
		if combatStart then
			local time = GetTime() - combatStart
			local msg = combatEndedString:format(math.floor(time / 60))
			for k, v in pairs(whisperers) do
				-- Notify people that combat has ended
				SendChatMessage(msg, "WHISPER", nil, k)
				whisperers[k] = nil
			end
			combatStart = nil
			if boss then
				frame:UnregisterEvent("UNIT_HEALTH")
				boss = nil
				bossHp = nil
			end
		end

		frame:UnregisterEvent("PLAYER_TARGET_CHANGED")
	elseif event == "UNIT_HEALTH" and arg1 and UnitName(arg1) == boss then
		bossHp = UnitHealth(arg1)
		-- Allow new status whispers every 10%
		if bossHp % 10 == 0 then
			for k in pairs(whisperers) do
				whisperers[k] = 1
			end
		end
	end
end)

frame:RegisterEvent("PLAYER_REGEN_DISABLED")
frame:RegisterEvent("PLAYER_REGEN_ENABLED")

